import os
from pytwss.mbean.clustermgr_mbean import getClusterMembers
from pytwss.mbean.nodeagent_mbean import getNodeAgentInfo
from pytwss.mbean.admin_server_mbean import getDeploymentMngrInfo
from pytwss.utils import log
import wsadmin_utils
import wsadmin_utils_cfg
from java.lang import String

#
# get the deployment information for the specified type
# param: type is "AG" or "SP" or "SPM"
#
def getDeploymentInfo(response_properties, type):
    target = response_properties.getProperty(type + "TargetScope")
    log("Getting deployment information for: %s=%s " %(type, target))
    if target:
       iSep = String(target).indexOf("/")
       if iSep >= 0:                        # node/appsrv
          node = String(target).substring(0, iSep)
          appsrv = String(target).substring(iSep+1)
          is_federated = wsadmin_utils_cfg.isFederated()
          __deploy_info = ServerInfo(node, appsrv, is_federated)
       elif target == "None":               # None constant
          __deploy_info = None
       else:                                # must be clustername
          clstr = target
          __deploy_info = ClusterInfo(clstr)
    else:
       __deploy_info = None                 # treat as None
    return __deploy_info

	
#
# The _DeployInfo object provides the interface for retrieving
# deployment information about the environment in which the user
# is connected.  
#
# This class and its subclasses rely on the wsadmin tooling.  The process
# that wsadmin connects to determines the configuration.  The process
# can be the deployment manager or a non-federated server.  The _DeployInfo class
# only retrieves the cell name and ids of the current deployment.  The
# recommend usage is to use a subclass, such as ClusterInfo or ServerInfo,
# to populate the deployment information based on the users configuration
# preference.
#
# known subclasses:  ClusterInfo, ServerInfo
class _DeployInfo:

	def __init__(self):	
		self.__cellName = wsadmin_utils.getCellName()
		self.__cellId = wsadmin_utils.getCellId(self.getCellName())	
		self.__clusterName = None
		self.__clusterId = None
		self.__nodeNames = None
		self.__nodeIds = None
		self.__agentIds = None
		self.__clusterMembers = None
		self.__nodeAgentInfo = None
		self.__dmInfo = None
	
	def getClusterName(self):
		return self.__clusterName		

	def getClusterId(self):
		return self.__clusterId
		
	def getCellName(self):
		return self.__cellName
		
	def getCellId(self):
		return self.__cellId
		
   	def getNodeNames(self):
		return self.__nodeNames
		
   	def getNodeIds(self):
		return self.__nodeIds 
		
   	def getAgentIds(self):
		return self.__agentIds 
   	   		
   	def getServerIds(self):		
		return self.__serverIds
		
	def getNodeAgentInfo(self):
		return self.__nodeAgentInfo

	def getDmId(self):
		return self.__dmId

	def getDmInfo(self):
		return self.__dmInfo

	def refreshNodeAgentInfo(self):
		return getNodeAgentInfo(self.getNodeNames())
		
	def getClusterMembers(self):
		return self.__clusterMembers
			
	def _setClusterName(self, clusterName):
		self.__clusterName = clusterName	

	def _setClusterId(self, clusterId):
		self.__clusterId = clusterId
		
	def _setCellName(self, cellName):
		self.__cellName = cellName
		
	def _setCellId(self, cellId):
		self.__cellId = cellId
		
   	def _setNodeNames(self, nodeNames):
		self.__nodeNames = nodeNames
		
   	def _setNodeIds(self, nodeIds):
		self.__nodeIds = nodeIds
   	   		
   	def _setServerIds(self, serverIds):		
		self.__serverIds = serverIds
		
   	def _setAgentIds(self, agentIds):		
		self.__agentIds = agentIds
   		
	def _setNodeAgentInfo(self, nodeAgentInfo):
		self.__nodeAgentInfo = nodeAgentInfo
	
	def _setDmInfo(self, dmInfo):
		self.__dmInfo = dmInfo	
	
	def _setDmId(self, dmId):
		self.__dmId = dmId	
			
	def _setClusterMembers(self, clusterMembers):
		self.__clusterMembers = clusterMembers	
	
	# Removes any duplicates from a list.
	# param: lst A list that possibly contains duplicates.
	# return: a new list with duplicates removed
	def _removeDups(self, lst):
		setList = []
		for item in lst:
			dupFound = 0
			for  it in setList:
				if(item == it):
					dupFound = 1
					break
			if(dupFound == 0):
				setList.append(item)
		return setList
	
	def toString(self):
		dep_info_str = []
		if self.__clusterName:
			dep_info_str.append("Cluster Name: %s" % self.__clusterName)
		if self.__clusterId:
			dep_info_str.append("Cluster Id: %s" % self.__clusterId)
		if self.__cellName:
			dep_info_str.append("Cell Name: %s" % self.__cellName)
		if self.__cellId:
			dep_info_str.append("Cell Id: %s" % self.__cellId )
		if self.__nodeNames:
			dep_info_str.append("Node Names: %s" % self.__nodeNames)
		if self.__nodeIds:
			dep_info_str.append("Node Ids: %s" % self.__nodeIds)
		if self.__agentIds:
			dep_info_str.append("Agent Ids: %s" % self.__agentIds)
		if self.__serverIds:
			dep_info_str.append("Server Ids: %s" % self.__serverIds)
		if self.__nodeAgentInfo:
			dep_info_str.append("Node Agent Info: [%s]" % self.__nodeAgentInfo.toString())
		if self.__dmInfo:
			dep_info_str.append("DM Info: [%s]" % self.__dmInfo.toString())												
		return os.linesep.join(dep_info_str)
#
# ClusterInfo populates the deployment information based on a cluster name.
#
# The cluster id will be made available as well as the node/server names and ids
# of the various cluster members.
#
# The NodeAgentInfo can also be queried to determine if each respective node
# agent in the cluster is running.
#
class ClusterInfo(_DeployInfo):
	#
	# Constructs a ClusterInfo object.
	# param: clusterName the name of the cluster 
	#
	def __init__(self, clusterName):
		_DeployInfo.__init__(self)
		if clusterName:
			log('Getting deployment info for cluster name %s' % clusterName)
			self._setClusterName(clusterName)
			self._setClusterId(wsadmin_utils.getClusterId(self.getClusterName()))
			self._setClusterMembers(getClusterMembers(self.getClusterName()))
			self._setNodeNames(self._removeDups([x.nodeName for x in self.getClusterMembers()]))
			self._setNodeIds([wsadmin_utils.getNodeId(x) for x in self.getNodeNames()])
			self._setAgentIds([wsadmin_utils.getServerId(x, 'nodeagent') for x in self.getNodeNames()])
			self._setServerIds([wsadmin_utils.getServerId(x.nodeName, x.memberName) for x in self.getClusterMembers()])
			dmInfo = getDeploymentMngrInfo()
			self._setDmInfo(dmInfo)
			dmId = wsadmin_utils.getServerId(dmInfo.getNodeName(), dmInfo.getServerName())
			self._setDmId(dmId)
			self._setNodeAgentInfo(getNodeAgentInfo(self.getNodeNames()))
		log('The deployment info is: %s' % self.toString())
#
# ServerInfo populates the deployment information based on a node name, an application
# server name, and whether or not the application server is federated to a deployment
# manager.
#
# The node and server id are retrievable at the first index of the lists
# returned by getNodeIds() and getServerIds().
#
# If the node has been federated to a deployment manager, the Deployment Manager
# info and the NodeAgentInfo can be queried through the appropriate getter methods.  
# However, if the server is standalone, these methods will return null.
#
class ServerInfo(_DeployInfo):
	#
	# Constructs a ServerInfo object.
	# param: nodeName the name of the node 
	# param: appSrvName the name of the application server 
	# param: isFederated 'true' or 'false'
	#	
	def __init__(self, nodeName, appSrvName, isFederated):
		_DeployInfo.__init__(self)
		log('Getting deployment info for server name %s, node name %s.' % (appSrvName, nodeName))
		if(appSrvName != None and nodeName != None):			
			self._setNodeNames([nodeName])
			self._setNodeIds([wsadmin_utils.getNodeId(nodeName)])
			self._setServerIds([wsadmin_utils.getServerId(nodeName, appSrvName)])
			        
		if(isFederated == "true"):
			log('The app server is federated.')
			dmInfo = getDeploymentMngrInfo()
			self._setDmInfo(dmInfo)
			dmId = wsadmin_utils.getServerId(dmInfo.getNodeName(), dmInfo.getServerName())
			self._setDmId(dmId)
			self._setNodeAgentInfo(getNodeAgentInfo(self.getNodeNames()))
			self._setAgentIds([wsadmin_utils.getServerId(nodeName, 'nodeagent')])
		else:
			log('The app server is a standalone server.')	
                        self._setDmInfo(None)
                        self._setDmId(None)
                        self._setNodeAgentInfo(None)
                        self._setAgentIds([])
				
		log('The deployment info is: %s' % self.toString())
				
if __name__ == '__main__':
	print '__main__'
